#!/usr/bin/python
# -*- coding: utf-8 -*-

'''pseudo Pound-Drever-Hall setup

                          -----
        (2*W_aom)-->(X)---|LPF|--->
                     |    -----     eps
                     O
                     |
                    ---
--------------------|\|-----------------(   )
 w_laser            ---                 cavity
                     |
                   -----
                   |AOM|
                   -----
                    _|_
                   ////

'''

from time import time
tic = time()

from sympy import *
from sympy.simplify.fu import *

init_printing()

#constants
E0, J0, J1, w_laser, w_pdh, t = symbols('E0, J0, J1, omega_laser, Omega_pdh, t', imaginary=False, real=True)
v_x, d_x, c = symbols('v_x, delta_x, c', imaginary=False, real=True)
w_aom = symbols('Omega_aom', imaginary=False, real=True)

#laser and phase-mod laser
E_laser = E0*exp(I*(w_laser*t))

#aom shifted phase-mod wave
E_aom =                                     \
  E0*exp(I*((w_laser+2*w_aom)*t))

#approximation of F(w) near a resonance
dnu, w_cav = symbols('delta_nu, omega_cav', imaginary=False, real=True)
def F(phi):
    dw = phi.diff(t) - w_cav
    return -(I/pi)*(dw/dnu)

#reflected phase-mod laser and dephased by doppler effect with double aom shift
dx = v_x*t + d_x
E_ref =                                                                                          \
   F( (w_laser)*t - (w_laser)*(dx)/(2*pi*c) ) * E0*exp( I*( (w_laser)*t - (w_laser)*(2*dx)/c ) )

#optical mixer
E_mich = sqrt(2)/2 * E_aom + sqrt(2)/2 * E_ref

#intensity of mixed wave
I_mich = abs(E_mich)**2
I_mich = expand(TR8(expand(expand_complex(I_mich))))

#Q demodulation of I_mich at 2*Omega_aom for doppler error signal obtention
kphi = symbols('k_phi', real=True)
eps = 2 * kphi * I_mich * cos(2*w_aom*t-pi/2)
eps = expand(TR8(TR7(expand(eps))))

toc = time()
print('Elapsed time : %fs'%(toc-tic))

'''results

eps =

       2               ⎛                  ω_laser   ⎞    ⎛2⋅δₓ⋅ω_laser   2⋅ω_laser⋅t⋅vₓ⎞
   2⋅E₀ ⋅J₀⋅J₁⋅k_φ_pdh⋅⎜ω_laser - ω_cav - ───────⋅vₓ⎟⋅cos⎜──────────── + ──────────────⎟
                       ⎝                     c      ⎠    ⎝     c               c       ⎠
 - ─────────────────────────────────────────────────────────────────────────────────────
                                  π⋅δ_ν

'''
